import { HazelReader, HazelWriter } from "@skeldjs/util";
import { Color, PlayerOutfitType } from "@skeldjs/constant";
import { Hostable } from "../Hostable";
import { GameData } from "../objects";
import { BaseRole } from "../roles";
export declare class TaskState {
    /**
     * The type of task that this task is, see {@link TaskType}. Not *necessarily*
     * available, although should be.
     */
    taskType: number;
    /**
     * Whether or not this task has been completed or not by the player.
     */
    completed: boolean;
    constructor(
    /**
     * The type of task that this task is, see {@link TaskType}. Not *necessarily*
     * available, although should be.
     */
    taskType: number, 
    /**
     * Whether or not this task has been completed or not by the player.
     */
    completed: boolean);
}
/**
 * Specifically represents a set of cosmetics for a player, this is used as a
 * layering system to also show what cosmetics the player if they have shapeshifted
 * into another player.
 */
export declare class PlayerOutfit {
    outfitType: PlayerOutfitType;
    name: string;
    color: Color;
    hatId: string;
    petId: string;
    skinId: string;
    visorId: string;
    nameplateId: string;
    constructor(outfitType: PlayerOutfitType, name: string, color: Color, hatId: string, petId: string, skinId: string, visorId: string, nameplateId: string);
    get isIncomplete(): boolean;
    static createDefault(outfitType: PlayerOutfitType): PlayerOutfit;
    static Deserialize(reader: HazelReader, type: PlayerOutfitType): PlayerOutfit;
    Serialize(writer: HazelWriter): void;
    clone(outfitType: PlayerOutfitType): PlayerOutfit;
}
export declare type PlayerOutfits = Partial<Record<PlayerOutfitType, PlayerOutfit>>;
/**
 * Represents information about a player, including any cosmetics they have equipped,
 * whether they are dead or the impostor, their role and whatever tasks they have.
 */
export declare class PlayerInfo<RoomType extends Hostable = Hostable> {
    readonly gamedata: GameData<RoomType>;
    readonly playerId: number;
    /**
     * A collection of this player's outfits as they should be displayed.
     * Somewhat of a layer system, the Default outfit shouldn't be displayed
     * if there is another outfit in this object.
     */
    outfits: PlayerOutfits;
    /**
     * This player's level/rank.
     */
    playerLevel: number;
    /**
     * Any flags that this player has, see {@link PlayerDataFlags}.
     */
    flags: number;
    /**
     * Which player this role is. Note that this is not their actual instance
     * of this role, see {@link PlayerData.role}, but is instead the class
     * used to create the instance, and holds metadata about the role, such as
     * the team it is for and whether it's a role for ghosts or not.
     *
     * You can use this to know what class {@link PlayerData.role} will be an
     * instance of.
     * @example
     * ```ts
     * if (player.playerInfo.roleType === ImpostorRole) {
     *   player.role instanceof ImpostorRole // true
     * }
     * ```
     */
    roleType: typeof BaseRole;
    /**
     * All of this player's tasks, and whether or not they have been completed
     * or not by the player.
     */
    taskStates: TaskState[];
    /**
     * The player's Innersloth friend code.
     */
    friendCode: string;
    /**
     * The player's global player UUID.
     */
    puid: string;
    /**
     * Create a default player info object.
     * @param gamedata The gamedata object that this player information belongs to.
     * @param playerId The ID of the player.
     * @returns A default player info object.
     */
    static createDefault<RoomType extends Hostable = Hostable>(gamedata: GameData<RoomType>, playerId: number): PlayerInfo<RoomType>;
    currentOutfitType: PlayerOutfitType;
    constructor(gamedata: GameData<RoomType>, playerId: number, 
    /**
     * A collection of this player's outfits as they should be displayed.
     * Somewhat of a layer system, the Default outfit shouldn't be displayed
     * if there is another outfit in this object.
     */
    outfits: PlayerOutfits, 
    /**
     * This player's level/rank.
     */
    playerLevel: number, 
    /**
     * Any flags that this player has, see {@link PlayerDataFlags}.
     */
    flags: number, 
    /**
     * Which player this role is. Note that this is not their actual instance
     * of this role, see {@link PlayerData.role}, but is instead the class
     * used to create the instance, and holds metadata about the role, such as
     * the team it is for and whether it's a role for ghosts or not.
     *
     * You can use this to know what class {@link PlayerData.role} will be an
     * instance of.
     * @example
     * ```ts
     * if (player.playerInfo.roleType === ImpostorRole) {
     *   player.role instanceof ImpostorRole // true
     * }
     * ```
     */
    roleType: typeof BaseRole, 
    /**
     * All of this player's tasks, and whether or not they have been completed
     * or not by the player.
     */
    taskStates: TaskState[], 
    /**
     * The player's Innersloth friend code.
     */
    friendCode: string, 
    /**
     * The player's global player UUID.
     */
    puid: string);
    static Deserialize<RoomType extends Hostable = Hostable>(reader: HazelReader, gamedata: GameData<RoomType>, playerId: number): PlayerInfo<RoomType>;
    /**
     * The player that this info is for.
     */
    getPlayer(): import("..").PlayerData<RoomType> | undefined;
    /**
     * Whether this player has been flagged as disconnected.
     */
    get isDisconnected(): boolean;
    /**
     * Whether this player has been flagged as the impostor.
     */
    get isImpostor(): boolean;
    /**
     * Whether this player has been flagged as dead.
     */
    get isDead(): boolean;
    get defaultOutfit(): PlayerOutfit;
    get currentOutfit(): PlayerOutfit;
    Deserialize(reader: HazelReader): void;
    Serialize(writer: HazelWriter): void;
    /**
     * Clone the player info for another player.
     */
    clone(playerId: number): PlayerInfo<RoomType>;
    /**
     * Set whether this player is flagged as disconnected.
     */
    setDisconnected(isDisconnected: boolean): void;
    /**
     * Set whether this player is flagged as an impostor.
     */
    setImpostor(isImpostor: boolean): void;
    /**
     * Set whether this player is flagged as dead.
     */
    setDead(isDead: boolean): void;
    /**
     * Create or overwrite one of this player's outfits.
     * @param outfit The outfit to set.
     */
    setOutfit(outfit: PlayerOutfit): void;
    /**
     * Get an outfit from this player, or create it from the player's default base
     * outfit if it doesn't exist, or create a new one entirely if that doesn't
     * exist.
     * @param outfitType The outfit to get.
     */
    getOutfit(outfitType: PlayerOutfitType): PlayerOutfit;
    /**
     * Delete an outfit on this player.
     * @param outfitType the outfit to delete.
     */
    deleteOutfit(outfitType: PlayerOutfitType): void;
    setName(outfitType: PlayerOutfitType, name: string): void;
    setColor(outfitType: PlayerOutfitType, color: Color): void;
    setHat(outfitType: PlayerOutfitType, hatId: string): void;
    setPet(outfitType: PlayerOutfitType, petId: string): void;
    setSkin(outfitType: PlayerOutfitType, skinId: string): void;
    setVisor(outfitType: PlayerOutfitType, visorId: string): void;
    setNameplate(outfitType: PlayerOutfitType, nameplateId: string): void;
    /**
     * Set the flags of this player.
     */
    setFlags(flags: number): void;
    /**
     * Set the tasks of this player.
     * @param taskIds The IDs of each task.
     */
    setTaskIds(taskIds: number[]): void;
    /**
     * Set the task states of this player.
     */
    setTaskStates(taskStates: TaskState[]): void;
    /**
     * Mark a task of this player as completed.
     */
    completeTask(state: TaskState): void;
}
//# sourceMappingURL=PlayerInfo.d.ts.map