import { SpawnType } from "@skeldjs/constant";
import { BaseRpcMessage } from "@skeldjs/protocol";
import { HazelReader, HazelWriter } from "@skeldjs/util";
import { ExtractEventTypes } from "@skeldjs/events";
import { Networkable, NetworkableEvents, NetworkableConstructor } from "../Networkable";
import { Hostable } from "../Hostable";
import { PlayerData } from "../PlayerData";
import { PlayerControl } from "./PlayerControl";
import { PlayerVoteState } from "../misc/PlayerVoteState";
import { PlayerInfo } from "../misc/PlayerInfo";
import { GameDataAddPlayerEvent, GameDataRemovePlayerEvent, GameDataSetTasksEvent } from "../events";
export interface GameDataData {
    players: Map<number, PlayerInfo>;
}
export declare type GameDataEvents<RoomType extends Hostable = Hostable> = NetworkableEvents<RoomType> & ExtractEventTypes<[
    GameDataAddPlayerEvent<RoomType>,
    GameDataRemovePlayerEvent<RoomType>,
    GameDataSetTasksEvent<RoomType>
]>;
export declare type PlayerIDResolvable = number | PlayerData | PlayerControl | PlayerInfo | PlayerVoteState;
/**
 * Represents a room object containing data about players.
 *
 * See {@link GameDataEvents} for events to listen to.
 */
export declare class GameData<RoomType extends Hostable = Hostable> extends Networkable<GameDataData, GameDataEvents<RoomType>, RoomType> implements GameDataData {
    /**
     * The players in the game data.
     */
    players: Map<number, PlayerInfo<RoomType>>;
    constructor(room: RoomType, spawnType: SpawnType, netId: number, ownerid: number, flags: number, data?: HazelReader | GameDataData);
    Awake(): void;
    getComponent<T extends Networkable>(component: NetworkableConstructor<T>): T | undefined;
    get owner(): RoomType;
    /**
     * Resolve some player identifier to a definite player info object.
     * @param resolvable The player identifier to resolve.
     * @returns A definite player info object.
     */
    resolvePlayerData(resolvable: PlayerIDResolvable): PlayerInfo<RoomType> | undefined;
    Deserialize(reader: HazelReader, spawn?: boolean): void;
    Serialize(writer: HazelWriter, spawn?: boolean): boolean;
    HandleRpc(rpc: BaseRpcMessage): Promise<void>;
    /**
     * Make the player data dirty and update on the next FixedUpdate.
     * @param resolvable The player to make dirty.
     */
    markDirty(resolvable: PlayerIDResolvable): void;
    private _handleSetTasks;
    private _setTasks;
    private _rpcSetTasks;
    /**
     * Set the tasks of a player.
     * @param player The player to set the tasks of.
     * @param taskTypes The task types to give to the player, see {@link TaskType}.
     * @example
     *```typescript
     * room.gamedata.setTasks(player, [
     *   TheSkeldTask.ReactorUnlockManifolds,
     *   TheSkeldTask.ElectricDownloadData,
     *   TheSkeldTask.ShieldsPrimeShields,
     *   TheSkeldTask.NavigationDownloadData
     * ]);
     * ```
     */
    setTasks(player: PlayerIDResolvable, taskTypes: number[]): Promise<void>;
    /**
     * Mark a player's task as complete.
     * @param resolvable The player of the tasks to mark complete.
     * @param taskIdx The index of the player's tasks to mark complete.
     * @example
     *```typescript
     * // Complete all of a player's tasks.
     * for (let i = 0; i < player.playerInfo.tasks.length; i++) {
     *   room.gamedata.completeTask(player, i);
     * }
     * ```
     */
    completeTask(resolvable: PlayerIDResolvable, taskIdx: number): void;
    /**
     * Add a player to player data.
     * @param playerId The player ID of the player to add.
     * @example
     *```typescript
     * // Get an available player ID and add it to the gamedata.
     * const playerId = room.getAvailablePlayerID();
     * room.gamedata.add(playerId);
     * ```
     */
    add(playerId: number): Promise<PlayerInfo<RoomType>>;
    /**
     * Remove player data from the game data.
     * @param resolvable The player to remove.
     */
    remove(resolvable: PlayerIDResolvable): Promise<void>;
}
//# sourceMappingURL=GameData.d.ts.map