"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomNetworkTransform = void 0;
const util_1 = require("@skeldjs/util");
const protocol_1 = require("@skeldjs/protocol");
const constant_1 = require("@skeldjs/constant");
const Networkable_1 = require("../../Networkable");
const net_1 = require("../../utils/net");
const events_1 = require("../../events");
/**
 * Represents player component for networking movement.
 *
 * See {@link CustomNetworkTransformEvents} for events to listen to.
 */
class CustomNetworkTransform extends Networkable_1.Networkable {
    constructor(room, spawnType, netId, ownerid, flags, data, playerControl) {
        super(room, spawnType, netId, ownerid, flags, data);
        this.oldSeqId || (this.oldSeqId = 0);
        this.seqId || (this.seqId = 0);
        this.position || (this.position = util_1.Vector2.null);
        this.velocity || (this.velocity = util_1.Vector2.null);
        this.player = this.owner;
        if (playerControl) {
            this.components = playerControl.components;
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn = false) {
        const newSeqId = reader.uint16();
        if (!net_1.NetworkUtils.seqIdGreaterThan(newSeqId, this.seqId)) {
            return;
        }
        this.seqId = newSeqId;
        this.position = reader.vector();
        this.velocity = reader.vector();
        this.emitSync(new events_1.PlayerMoveEvent(this.room, this.player, new util_1.Vector2(this.position), new util_1.Vector2(this.velocity)));
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn = false) {
        writer.uint16(this.seqId);
        writer.vector(this.position);
        writer.vector(this.velocity);
        this.dirtyBit = 0;
        return true;
    }
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.SnapTo:
                await this._handleSnapTo(rpc);
                break;
        }
    }
    /**
     * Move to a position (lerps towards).
     * @param position The position to move towards.
     * @param velocity The velocity to display moving at.
     * @example
     *```typescript
     * // Follow the host
     * host.transform.on("player.move", ev => {
     *   player.transform.move(ev.position.x, ev.position.y);
     * });
     * ```
     */
    async move(x, y, velocity = util_1.Vector2.null) {
        this.seqId += 1;
        if (this.seqId > 2 ** 16 - 1) {
            this.seqId = 1;
        }
        this.position.x = x;
        this.position.y = y;
        this.velocity = velocity;
        this.dirtyBit = 1;
        const writer = util_1.HazelWriter.alloc(10);
        this.Serialize(writer, false);
        await this.room.broadcast([new protocol_1.DataMessage(this.netId, writer.buffer)]);
        this.emitSync(new events_1.PlayerMoveEvent(this.room, this.player, new util_1.Vector2(this.position), new util_1.Vector2(this.velocity)));
    }
    async _handleSnapTo(rpc) {
        if (net_1.NetworkUtils.seqIdGreaterThan(rpc.sequenceid, this.seqId)) {
            const oldPosition = this.position;
            this.seqId = rpc.sequenceid;
            this.position = rpc.position;
            this.velocity = util_1.Vector2.null;
            const newPosition = new util_1.Vector2(this.position);
            const ev = await this.emit(new events_1.PlayerSnapToEvent(this.room, this.player, rpc, oldPosition, newPosition));
            if (ev.alteredPosition.x !== newPosition.x ||
                ev.alteredPosition.y !== newPosition.y) {
                this.position = new util_1.Vector2(ev.alteredPosition);
                this._rpcSnapTo(ev.alteredPosition);
            }
        }
    }
    _snapTo(position) {
        this.position = position;
    }
    _rpcSnapTo(position) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.SnapToMessage(new util_1.Vector2(position), this.seqId)));
    }
    async snapTo(x, y, rpc) {
        var _a;
        this.seqId += 1;
        if (this.seqId > 2 ** 16 - 1) {
            this.seqId = 1;
        }
        this.dirtyBit = 0;
        const oldPosition = this.position;
        if (typeof x === "number") {
            return this.snapTo(new util_1.Vector2(x, y || 0), rpc !== null && rpc !== void 0 ? rpc : true);
        }
        this._snapTo(x);
        const ev = await this.emit(new events_1.PlayerSnapToEvent(this.room, this.player, undefined, oldPosition, x));
        if (ev.alteredPosition.x !== x.x ||
            ev.alteredPosition.y !== x.y) {
            this.position = new util_1.Vector2(ev.alteredPosition);
        }
        if ((_a = rpc !== null && rpc !== void 0 ? rpc : y) !== null && _a !== void 0 ? _a : true) {
            this._rpcSnapTo(this.position);
        }
    }
}
exports.CustomNetworkTransform = CustomNetworkTransform;
//# sourceMappingURL=CustomNetworkTransform.js.map