"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VoteBanSystem = void 0;
const constant_1 = require("@skeldjs/constant");
const protocol_1 = require("@skeldjs/protocol");
const Networkable_1 = require("../../Networkable");
/**
 * Represents a room object for handling vote kicks.
 *
 * See {@link VoteBanSystemEvents} for events to listen to.
 */
class VoteBanSystem extends Networkable_1.Networkable {
    constructor(room, spawnType, netId, ownerid, flags, data, gameData) {
        super(room, spawnType, netId, ownerid, flags, data);
        this.voted || (this.voted = new Map);
        if (gameData) {
            this.components = gameData.components;
        }
    }
    get owner() {
        return super.owner;
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn = false) {
        const num_players = reader.upacked();
        for (let i = 0; i < num_players; i++) {
            const clientId = reader.uint32();
            if (this.voted.get(clientId)) {
                this.voted.set(clientId, [undefined, undefined, undefined]);
            }
            this.voted.set(clientId, [undefined, undefined, undefined]);
            for (let i = 0; i < 3; i++) {
                reader.upacked();
            }
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn = false) {
        writer.upacked(this.voted.size);
        for (const [clientId, voters] of this.voted) {
            writer.uint32(clientId);
            for (let i = 0; i < 3; i++) {
                if (voters[i])
                    writer.upacked(voters[i].clientId);
            }
        }
        return true;
    }
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.AddVote:
                this._handleAddVote(rpc);
                break;
        }
    }
    _handleAddVote(rpc) {
        const voting = this.room.players.get(rpc.votingid);
        const target = this.room.players.get(rpc.targetid);
        if (voting && target) {
            this._addVote(voting, target);
        }
    }
    _addVote(voter, target) {
        const voted = this.voted.get(target.clientId);
        if (voted) {
            const next = voted.indexOf(undefined);
            if (~next) {
                voted[next] = voter;
                this.dirtyBit = 1;
            }
            if (this.room.hostIsMe && voted.every((v) => v !== null)) {
                this.room.broadcast([], [
                    new protocol_1.KickPlayerMessage(this.room.code, target.clientId, false, constant_1.DisconnectReason.Error),
                ]);
            }
        }
        else {
            this.voted.set(target.clientId, [voter, undefined, undefined]);
            this.dirtyBit = 1;
        }
    }
    _rpcAddVote(voter, target) {
        this.room.messageStream.push(new protocol_1.RpcMessage(this.netId, new protocol_1.AddVoteMessage(voter.clientId, target.clientId)));
    }
    /**
     * Add the vote of a player to vote for another player to be kicked.
     * @param voter The player to count the vote as.
     * @param target The player to vote to be kicked.
     * @example
     *```typescript
     * room.votebansystem.addVote(client.me, player);
     * ```
     */
    addVote(voter, target) {
        const _voter = this.room.resolvePlayer(voter);
        const _target = this.room.resolvePlayer(target);
        if (_voter && _target) {
            this._addVote(_voter, _target);
            this._rpcAddVote(_voter, _target);
        }
    }
}
exports.VoteBanSystem = VoteBanSystem;
//# sourceMappingURL=VoteBanSystem.js.map